package pl.nettigo.esp.configurator;

import android.content.Intent;
import android.os.Bundle;
import android.support.v7.app.AppCompatActivity;
import android.view.View;
import android.widget.Button;
import android.widget.TextView;

import java.net.InetAddress;
import java.net.SocketException;

import pl.nettigo.esp.configurator.exception.UnableToStartUdpServerException;
import pl.nettigo.esp.configurator.model.Command;
import pl.nettigo.esp.configurator.util.UdpMessenger;

public class DeviceSetupActivity extends AppCompatActivity {

    private TextView mDeviceInfo;
    private String mSerialNumber;
    private InetAddress mIpAddress;
    private Button mSetWifiButton;
    private Button mDoRebootButton;
    private UdpMessenger mUdpMessenger;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_device_setup);
        mDeviceInfo = findViewById(R.id.label_device_info);
        mSetWifiButton = findViewById(R.id.device_setup_network);
        mDoRebootButton = findViewById(R.id.device_reboot);

        try {
            mUdpMessenger = UdpMessenger.getInstance();
        } catch (SocketException e) {
            throw new UnableToStartUdpServerException();
        }

        Intent intent = getIntent();

        mSerialNumber = intent.getStringExtra(MainActivity.EXTRA_SERIAL_NUMBER);
        mIpAddress = (InetAddress) intent.getSerializableExtra(MainActivity.EXTRA_IP_ADDRESS);

        if(mSerialNumber == null || mIpAddress == null) {
            throw new RuntimeException("Do not run this activity without proper intent!");
        }

        mDeviceInfo.setText(getString(R.string.label_device_info, mSerialNumber, mIpAddress.toString().replace("/","")));

        mSetWifiButton.setOnClickListener(new Button.OnClickListener() {
            @Override
            public void onClick(View view) {
                onSetWifiButtonClick();
            }
        });

        mDoRebootButton.setOnClickListener(new Button.OnClickListener() {
            @Override
            public void onClick(View view) {
                onDoRebootButtonClick();
            }
        });
    }

    @Override
    protected void onResume() {
        super.onResume();
        mUdpMessenger.setTargetAddress(mIpAddress);
    }

    public void onSetWifiButtonClick() {
        Intent intent = new Intent(DeviceSetupActivity.this, NetworkSetupActivity.class);
        intent.putExtra(MainActivity.EXTRA_IP_ADDRESS, mIpAddress);
        startActivity(intent);
    }

    public void onDoRebootButtonClick() {
        Command command = new Command("doReboot");
        String data = UdpMessenger.buildJsonString(command);
        mUdpMessenger.sendData(data);
        finish();
    }
}
