package pl.nettigo.esp.configurator;

import android.content.Intent;
import android.os.Bundle;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.LinearLayoutManager;
import android.support.v7.widget.RecyclerView;
import android.view.View;
import android.widget.Button;

import com.google.gson.Gson;
import com.google.gson.reflect.TypeToken;

import java.io.IOException;
import java.lang.reflect.Type;
import java.net.DatagramPacket;
import java.net.InetAddress;
import java.net.SocketException;

import pl.nettigo.esp.configurator.adapter.DevicesAdapter;
import pl.nettigo.esp.configurator.exception.UnableToStartUdpServerException;
import pl.nettigo.esp.configurator.model.BaseResponse;
import pl.nettigo.esp.configurator.model.Command;
import pl.nettigo.esp.configurator.model.EspDevice;
import pl.nettigo.esp.configurator.model.Response;
import pl.nettigo.esp.configurator.util.UdpMessenger;

public class MainActivity extends AppCompatActivity implements DevicesAdapter.OnSetupButtonClickListener, UdpMessenger.OnPacketReceiveListener {

    public static final String EXTRA_SERIAL_NUMBER = "pl.nettigo.esp.configurator.SERIAL_NUMBER";
    public static final String EXTRA_IP_ADDRESS = "pl.nettigo.esp.configurator.IP_ADDRESS";

    private RecyclerView mDevicesRecycler;
    private DevicesAdapter mAdapter = new DevicesAdapter();
    private Button mScanDevicesButton;
    private UdpMessenger mUdpMessenger;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);
        mDevicesRecycler = findViewById(R.id.devices_list);
        mScanDevicesButton = findViewById(R.id.devices_scan);

        try {
            mUdpMessenger = UdpMessenger.getInstance();
        } catch (SocketException e) {
            throw new UnableToStartUdpServerException();
        }

        mDevicesRecycler.setHasFixedSize(false);

        RecyclerView.LayoutManager layoutManager = new LinearLayoutManager(this);
        mDevicesRecycler.setLayoutManager(layoutManager);


        mAdapter.setOnSetupButtonClickListener(this);
        mDevicesRecycler.setAdapter(mAdapter);

        mScanDevicesButton.setOnClickListener(new Button.OnClickListener() {
            @Override
            public void onClick(View view) {
                onScanDevicesButtonClick();
            }
        });
    }

    @Override
    protected void onResume() {
        super.onResume();
        mAdapter.clearList();
        mUdpMessenger.setOnPacketReceiveListener(this);
    }

    @Override
    public void onSetupButtonClick(EspDevice device) {
        Intent intent = new Intent(this, DeviceSetupActivity.class);
        intent.putExtra(EXTRA_SERIAL_NUMBER, device.getSerialNumber());
        intent.putExtra(EXTRA_IP_ADDRESS, device.getIpAddress());
        startActivity(intent);
    }

    public void onScanDevicesButtonClick() {
        Command command = new Command("getDeviceInfo");
        try {
            mAdapter.clearList();
            InetAddress address = UdpMessenger.getBroadcastAddress(this);
            String data = UdpMessenger.buildJsonString(command);
            mUdpMessenger.sendData(address, data);
        } catch (IOException ignored) {
        }
    }

    @Override
    public void onPacketReceive(DatagramPacket packet) {
        String json = UdpMessenger.parseDatagramPacket(packet);
        BaseResponse data = new Gson().fromJson(json, BaseResponse.class);
        if(data.getType().equals("deviceInfo")) {
            Type responseType = new TypeToken<Response<EspDevice>>() {
            }.getType();
            final Response<EspDevice> targetObject = new Gson().fromJson(json, responseType);
            targetObject.getResult().setIpAddress(packet.getAddress());
            runOnUiThread(new Runnable() {
                public void run() {
                    mAdapter.addDevice(targetObject.getResult());
                }
            });

        }
    }
}
