package pl.nettigo.esp.configurator.adapter;

import android.content.Context;
import android.graphics.drawable.Drawable;
import android.support.v7.widget.RecyclerView;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.ImageView;
import android.widget.TextView;

import java.util.LinkedList;
import java.util.List;

import pl.nettigo.esp.configurator.R;
import pl.nettigo.esp.configurator.model.WirelessNetwork;

public class NetworksAdapter extends RecyclerView.Adapter<NetworksAdapter.ViewHolder> {
    private List<WirelessNetwork> mDataset;
    private OnWirelessNetworkItemClickListener onWirelessNetworkItemClickListener = null;
    private Context mContext;

    public static class ViewHolder extends RecyclerView.ViewHolder {
        public View mItem;
        public TextView mSsid;
        public ImageView mSignalStrength;

        public ViewHolder(View itemView) {
            super(itemView);
            mItem = itemView.findViewById(R.id.network_item);
            mSsid = itemView.findViewById(R.id.network_ssid);
            mSignalStrength = itemView.findViewById(R.id.network_signal);
        }
    }

    public NetworksAdapter(Context context, List<WirelessNetwork> networks) {
        this.mContext = context;
        this.mDataset = new LinkedList<>(networks);
    }

    public NetworksAdapter(Context context) {
        this.mContext = context;
        this.mDataset = new LinkedList<>();
    }

    public void setOnWirelessNetworkItemClickListener(OnWirelessNetworkItemClickListener onWirelessNetworkItemClickListener) {
        this.onWirelessNetworkItemClickListener = onWirelessNetworkItemClickListener;
    }

    @Override
    public ViewHolder onCreateViewHolder(ViewGroup parent, int viewType) {
        View view = LayoutInflater.from(parent.getContext()).inflate(R.layout.viewholder_network, parent, false);
        return new ViewHolder(view);
    }

    @Override
    public void onBindViewHolder(ViewHolder holder, int position) {
        final WirelessNetwork network = mDataset.get(position);

        holder.mSsid.setText(network.getSSID());

        Drawable wifiInfoIconDrawable;
        Integer signalQuality = network.getSignalQuality();
        if(network.isProtected()) {
            if(signalQuality >= 75) {
                wifiInfoIconDrawable = mContext.getResources().getDrawable(R.drawable.ic_signal_wifi_4_bar_lock_black_24dp);
            } else if(signalQuality >= 50) {
                wifiInfoIconDrawable = mContext.getResources().getDrawable(R.drawable.ic_signal_wifi_3_bar_lock_black_24dp);
            } else if(signalQuality >= 25) {
                wifiInfoIconDrawable = mContext.getResources().getDrawable(R.drawable.ic_signal_wifi_2_bar_lock_black_24dp);
            } else {
                wifiInfoIconDrawable = mContext.getResources().getDrawable(R.drawable.ic_signal_wifi_1_bar_lock_black_24dp);
            }
        } else {
            if(signalQuality >= 75) {
                wifiInfoIconDrawable = mContext.getResources().getDrawable(R.drawable.ic_signal_wifi_4_bar_black_24dp);
            } else if(signalQuality >= 50) {
                wifiInfoIconDrawable = mContext.getResources().getDrawable(R.drawable.ic_signal_wifi_3_bar_black_24dp);
            } else if(signalQuality >= 25) {
                wifiInfoIconDrawable = mContext.getResources().getDrawable(R.drawable.ic_signal_wifi_2_bar_black_24dp);
            } else {
                wifiInfoIconDrawable = mContext.getResources().getDrawable(R.drawable.ic_signal_wifi_1_bar_black_24dp);
            }
        }

        holder.mSignalStrength.setImageDrawable(wifiInfoIconDrawable);

        holder.mItem.setOnClickListener(new Button.OnClickListener() {
            @Override
            public void onClick(View view) {
                if (onWirelessNetworkItemClickListener != null) {
                    onWirelessNetworkItemClickListener.onWirelessNetworkItemClick(network);
                }
            }
        });
    }

    @Override
    public int getItemCount() {
        return mDataset.size();
    }

    public void clearList() {
        mDataset.clear();
        notifyDataSetChanged();
    }

    public boolean addNetwork(WirelessNetwork network) {
        boolean result = mDataset.add(network);
        notifyItemInserted(mDataset.size() - 1);
        return result;
    }

    public void removeNetwork(WirelessNetwork network) {
        int position = mDataset.indexOf(network);
        mDataset.remove(position);
        notifyItemRemoved(position);
    }

    public interface OnWirelessNetworkItemClickListener {
        void onWirelessNetworkItemClick(WirelessNetwork network);
    }
}
